package me.teridax.jcash;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.logging.*;

public final class Logging {

    public static final Logger LOGGER = Logger.getLogger(Logging.class.getName());

    private static final String LOG_FOLDER_NAME = "logs/";

    /**
     * Initialize the global system logger.
     * Adds a file logging handler
     */
    static void initializeSystemLogger(Level level) {
        LogManager.getLogManager().reset();

        createConsoleLogger(level);
        createFileLogger(level);

        LOGGER.setLevel(level);
    }

    private static void createConsoleLogger(Level level) {
        var ch = new ConsoleHandler();
        ch.setLevel(level);
        LOGGER.addHandler(ch);
    }

    private static void createFileLogger(Level level) {
        var now = LocalDateTime.now();
        var dateTime = DateTimeFormatter.ISO_DATE_TIME.format(now);
        var logFileName = LOG_FOLDER_NAME + dateTime + ".log";

        initializeLogFolder();

        try {
            var fh = new FileHandler(logFileName);
            fh.setLevel(level);
            LOGGER.addHandler(fh);
        } catch (IOException e) {
            LOGGER.warning("Unable to initialize logging for file: " + logFileName);
        }
    }

    private static void initializeLogFolder() {
        var folderPath = Path.of(LOG_FOLDER_NAME);

        if (Files.isDirectory(folderPath))
            return;

        try {
            Files.createDirectory(folderPath);
        } catch (IOException e) {
            LOGGER.warning("Unable to create directory: " + folderPath);
        }
    }
}
