package me.teridax.jcash.gui.login;

import javax.swing.*;
import javax.swing.text.NumberFormatter;
import java.awt.*;
import java.text.NumberFormat;

public class LoginView extends JPanel {

    private JFormattedTextField blz;
    private JFormattedTextField iban;
    private JPasswordField pin;
    private JButton login;

    public LoginView() {
        createComponents();
        layoutComponents();
    }

    private void layoutComponents() {
        var content = new JPanel(new GridBagLayout());

        this.setBorder(BorderFactory.createEmptyBorder(8,8,8,8));
        this.setLayout(new BorderLayout(16, 16));
        this.add(new JScrollPane(content), BorderLayout.CENTER);
        this.add(new JLabel("Bankautomat"), BorderLayout.NORTH);

        var constraints = new GridBagConstraints();

        constraints.gridwidth = 4;
        constraints.insets = new Insets(12,12,12,12);

        addInputRow(constraints, content, blz, 1, "BLZ");
        addInputRow(constraints, content, iban, 2, "IBAN");
        addInputRow(constraints, content, pin, 3, "PIN");

        constraints.gridy = 4;
        constraints.anchor = GridBagConstraints.PAGE_END;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.NONE;
        constraints.insets = new Insets(12,12,12,12);
        content.add(login, constraints);
    }

    private void createComponents() {
        this.blz = new JFormattedTextField();
        this.iban = new JFormattedTextField(getNumberFormat());
        this.pin = new JPasswordField();
        this.login = new JButton("Login");
    }

    private NumberFormatter getNumberFormat() {
        var format = NumberFormat.getIntegerInstance();
        format.setGroupingUsed(false);

        var formatter = new NumberFormatter(format);
        formatter.setValueClass(Integer.class);
        formatter.setMinimum(0);
        formatter.setMaximum(Integer.MAX_VALUE);
        formatter.setAllowsInvalid(false);

        return formatter;
    }

    private void addInputRow(GridBagConstraints constraints, JComponent target, JComponent comp, int row, String name) {
        constraints.gridwidth = 1;
        constraints.gridx = 1;
        constraints.gridy = row;
        constraints.weightx = 0;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        target.add(new JLabel(name, SwingConstants.RIGHT), constraints);

        constraints.gridx = 2;
        constraints.gridy = row;
        constraints.weightx = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        target.add(comp, constraints);
    }

    public JTextField getBlz() {
        return blz;
    }

    public JTextField getIban() {
        return iban;
    }

    public JPasswordField getPin() {
        return pin;
    }

    public JButton getLogin() {
        return login;
    }
}
