package me.teridax.jcash.gui.transfer;

import me.teridax.jcash.Logging;
import me.teridax.jcash.decode.StringDecoder;

import javax.swing.*;
import java.awt.*;

import static javax.swing.JOptionPane.ERROR_MESSAGE;
import static javax.swing.JOptionPane.showMessageDialog;

public class TransferView {

    private JDialog dialog;
    private JButton cancel;
    private JButton transfer;
    private JFormattedTextField iban;
    private JFormattedTextField blz;
    private JFormattedTextField value;

    public TransferView() {
        createComponents();
        layoutComponents();
    }

    public void showDialog() {
        dialog.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
        dialog.setTitle("Transfer money");
        dialog.pack();
        dialog.setSize(dialog.getWidth() * 2, dialog.getHeight());
        dialog.setResizable(false);
        dialog.setLocationRelativeTo(null);
        dialog.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        dialog.setVisible(true);
    }

    private void layoutComponents() {
        var c = new GridBagConstraints();

        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        c.weighty = 1;
        c.gridwidth = 3;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(4, 4, 4, 4);
        dialog.getContentPane().add(new JLabel("Transfer money"), c);

        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 1;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 0;
        dialog.getContentPane().add(new JLabel("BLZ", SwingConstants.RIGHT), c);

        c.gridx = 1;
        c.gridy = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 0.5;
        dialog.getContentPane().add(blz, c);

        c.gridx = 2;
        c.gridy = 1;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 0;
        dialog.getContentPane().add(new JLabel("IBAN", SwingConstants.RIGHT), c);

        c.gridx = 3;
        c.gridy = 1;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 1;
        dialog.getContentPane().add(iban, c);

        c.gridx = 0;
        c.gridy = 2;
        c.fill = GridBagConstraints.NONE;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 0;
        dialog.getContentPane().add(new JLabel("Betrag", SwingConstants.RIGHT), c);

        c.gridx = 1;
        c.gridy = 2;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.weightx = 0.5;
        dialog.getContentPane().add(value, c);

        var buttonPanel = new JPanel(new FlowLayout(FlowLayout.TRAILING));
        buttonPanel.add(cancel);
        buttonPanel.add(transfer);

        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth = 4;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.LAST_LINE_END;
        c.insets = new Insets(10, 10, 10, 10);
        dialog.getContentPane().add(buttonPanel, c);
    }

    private void createComponents() {
        this.dialog = new JDialog();

        this.cancel = new JButton("Cancel");
        this.transfer = new JButton("Transfer");
        this.value = new JFormattedTextField(StringDecoder.LOCAL_NUMBER_FORMAT);
        this.iban = new JFormattedTextField();
        this.blz = new JFormattedTextField();

        this.dialog.setContentPane(new JPanel(new GridBagLayout()));
    }

    public double getAmount() {
        if (value.getText().isBlank()) {
            Logging.LOGGER.severe("Amount is empty");
            showMessageDialog(null, "invalid amount", "currency must not be blank", ERROR_MESSAGE);
            return 0;
        }

        try {
            return StringDecoder.decodeCurrency(value.getText());
        } catch (IllegalArgumentException e) {
            Logging.LOGGER.severe("Invalid amount: " + value.getText());
            throw new RuntimeException(e);
        }
    }

    public JButton getCancel() {
        return cancel;
    }

    public JButton getTransfer() {
        return transfer;
    }

    public String getIban() {
        return iban.getText();
    }

    public String getBlz() {
        return blz.getText();
    }

    public void dispose() {
        this.dialog.dispose();
    }
}
